package com.acme.ejb;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.util.logging.Logger;

import javax.ejb.embeddable.EJBContainer;
import javax.naming.*;
import javax.validation.*;

import org.glassfish.embeddable.*;
import org.glassfish.embeddable.archive.*;
import org.glassfish.embeddable.archive.ScatteredArchive.Type;
import org.junit.*;
import org.junit.rules.*;

public class GreetingManagerTest
{
	
	@Rule
	public TestName testMethod = new TestName();
	
	private static final Logger logger = Logger.getLogger(GreetingManagerTest.class.getName());

	/**
	 * This test case is meant to succeed despite the presence of META-INF/validation.xml in target/classes.
	 * Arquillian should work in this manner. This test uses the embeddable EJB container API of EJB 3.1.
	 * @throws Exception Any exception encountered during test execution
	 */
	@Test
	public void testGreetingWithEJBContainer() throws Exception
	{
		logger.info("Starting execution of test" + testMethod.getMethodName());
		
		EJBContainer container = null;
		try
		{
			//Setup
			container = EJBContainer.createEJBContainer();
			Context context = container.getContext();
			GreetingManager manager = (GreetingManager) context.lookup("java:global/glassfish-ejb-validation/GreetingManager");
			
			//Execute
			String greeting = manager.greet("Earthlings");
			
			//Also print out ValidatorFactory and Validator classes.
			ValidatorFactory validatorFactory = Validation.buildDefaultValidatorFactory();
			Validator validator = validatorFactory.getValidator();
			logger.info("ValidatorFactory class is " + validatorFactory.getClass().getName());
			logger.info("Validator class is " + validator.getClass().getName());
			
			//Verify
			assertEquals("Hello Earthlings", greeting);
		}
		finally
		{
			if(container != null)
			{
				container.close();
			}
			logger.info("Ending execution of test" + testMethod.getMethodName());
		}
	}
	
	@Test
	public void testGreetingWithGlassfishRuntime() throws Exception
	{
		logger.info("Starting execution of test" + testMethod.getMethodName());
		
		GlassFish glassFish = null;
		Deployer deployer = null;
		String appName = null;
		try
		{
			//Setup
			BootstrapProperties bootstrapProps = new BootstrapProperties();
			GlassFishRuntime glassFishRuntime = GlassFishRuntime.bootstrap(bootstrapProps);
			
			GlassFishProperties gfProps = new GlassFishProperties();
			
			glassFish = glassFishRuntime.newGlassFish(gfProps);
			glassFish.start();
			
			deployer = glassFish.getDeployer();
			ScatteredArchive archive = new ScatteredArchive("glassfish-ejb-validation", Type.JAR);
			archive.addClassPath(new File("target", "classes"));
			archive.addClassPath(new File("target", "test-classes"));
			
			appName = deployer.deploy(archive.toURI(), "--force=true");
			
			InitialContext context = new InitialContext();
			
			//Execute
			GreetingManager manager = (GreetingManager) context.lookup("java:global/glassfish-ejb-validation/GreetingManager");
			String greeting = manager.greet("Earthlings");
			
			//Also print out ValidatorFactory and Validator classes.
			ValidatorFactory validatorFactory = Validation.buildDefaultValidatorFactory();
			Validator validator = validatorFactory.getValidator();
			logger.info("ValidatorFactory class is " + validatorFactory.getClass().getName());
			logger.info("Validator class is " + validator.getClass().getName());
			
			//Verify
			assertEquals("Hello Earthlings", greeting);
		}
		finally
		{
			if(deployer != null && appName != null)
			{
				deployer.undeploy(appName);
			}
			if(glassFish != null)
			{
				glassFish.stop();
				glassFish.dispose();
			}
			logger.info("Ending execution of test" + testMethod.getMethodName());
		}
	}
}
